def reduce(merge, collection, base):
    """reduce

    Args:
        merge (function): a function that merges two objects
        collection (list): a list of objects
        base (object): a base case object
    Returns:
        a reduced object
    """
    def tail(xs): return xs[1:]

    if len(collection) == 0:
        return base
    else:
        return reduce(merge, tail(collection), merge(base, collection[0]))



def check_input(argd):
    """Check whether the user input is valid

    Args:
        argd (dict): user input dict
    Returns:
        same as input if no errors found
        otherwise exit with a message.
    """
    for k in ['const', 'rules', 'tasks']:
        if k not in argd:
            print(
                "Error hint: missing required field: \"{}\"".format(k))
            exit()
    return argd



import sys
import os
from .core import ninja_generate
from .tk import ioTK
from .io.read import read_config_files


def main(f_out, args):
    if len(args) == 0:
        here = os.path.dirname(os.path.realpath(__file__))
        f_usage = os.path.join(here, 'usage.txt')
        print(ioTK.read_all(f_usage))
        print("Error hint: need at least two command line arguments")
        exit()

    ioTK.save_text(
            f_out,
            ninja_generate(read_config_files(args))
        )



from .ninja_const import ninja_const


def ninja_build(tasks):
    """Return the Ninja build section

    Args:
        tasks (list): a list of task dictionaries
    Returns:
        a string
    """
    output = []
    for t in tasks:
        item = "build {}: {} {}".format(
                    " ".join(t['out']),
                    t['rule'],
                    " ".join(t['in'])
                )
        if 'const' in t:
            output.append("{}\n{}".format(
                    item,
                    ninja_const(t['const'], "  "))
                )
        else:
            output.append(item)
    return "\n\n".join(output)



"""
Ninjag: Ninja build file generator

Usage: ninjag output.ninja in1.yaml in2.yaml ...
Author: Steven(Yuhang) Wang
License: MIT/X11
"""
import sys
from .main import main


def cli():
    main(sys.argv[1], sys.argv[2:])



def ninja_const(inputDict, indent=""):
    """Return a list of user defined constants

    Args:
        inputDict (dict): a dictionary of constant definitions
        indent (str): indentation for each definition
    Returns:
        a string
    """
    output = []
    for k, v in inputDict.items():
        output.append(
                "{}{} = {}".format(indent, k, v)
            )
    return "\n\n".join(output)
